<?php

declare(strict_types=1);

namespace Drupal\mcp_client\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mcp_client\Entity\McpServer;
use Drupal\mcp_client\MCPClient;

/**
 * MCP Server form.
 */
final class McpServerForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
      '#description' => $this->t('Label for the server for internal usage. Data name will prefix all tools.'),
      '#attributes' => [
        'placeholder' => 'My MCP Server',
      ],
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [McpServer::class, 'load'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description'),
      '#description' => $this->t('A description of the server for internal purposes.'),
      '#rows' => 2,
    ];

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hostname'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->get('endpoint'),
      '#required' => TRUE,
      '#description' => $this->t('The endpoint of the server.'),
      '#attributes' => [
        'placeholder' => 'https://example.com',
      ],
    ];

    $tools_available = $this->entity->get('tools') ?? [];
    $tools_to_pick = [];
    foreach ($tools_available as $tool) {
      $tools_to_pick[$tool['name']] = $tool['name'] . ' (' . $tool['description'] . ')';
    }

    $form['enabled_tools'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled tools'),
      '#description' => $this->t('Select the tools you want to enable.'),
      '#options' => $tools_to_pick,
      '#default_value' => $this->entity->get('enabled_tools') ?? [],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $client = new MCPClient(
      $this->entity->get('endpoint'),
    );
    $tools = $client->listTools();
    if (count($tools) > 0) {
      // Set the tool name as key.
      $save_tools = [];
      foreach ($tools as $key => $tool) {
        $save_tools[$tool['name']] = $tool;
      }
      $this->entity->set('tools', $save_tools);
    }

    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    // Clear the plugin cache for AIFucntionCalls.
    \Drupal::service('plugin.manager.ai.function_calls')->clearCachedDefinitions();
    return $result;
  }

}
