<?php

declare(strict_types=1);

namespace Drupal\mcp_client;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * A generic MCP client that can work with vanilla PHP.
 */
class MCPClient {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * The server endpoint URL.
   *
   * @var string
   */
  protected string $endpoint;

  /**
   * Constructor.
   *
   * @param string $endpoint
   *   The MCP server endpoint URL.
   * @param \GuzzleHttp\Client|null $http_client
   *   The HTTP client, or NULL to create one.
   */
  public function __construct(string $endpoint, ?Client $http_client = NULL) {
    $this->endpoint = rtrim($endpoint, '/');
    $this->httpClient = $http_client ?? new Client();
  }

  /**
   * Lists all available tools on the MCP server.
   *
   * @return array
   *   An array of tool information.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function listTools(): array {
    try {
      $response = $this->httpClient->get("{$this->endpoint}/tools");
      // We want to split this out into pure PHP in the future, thus not JSON.
      $data = json_decode($response->getBody()->getContents(), TRUE);

      return $data['tools'] ?? [];
    }
    catch (GuzzleException $e) {
      throw $e;
    }
  }

  /**
   * Executes a specific tool on the MCP server.
   *
   * @param string $toolId
   *   The ID of the tool to execute.
   * @param array $parameters
   *   The parameters to pass to the tool.
   *
   * @return array
   *   The tool execution result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function executeTool(string $toolId, array $parameters = []): array {
    try {
      $response = $this->httpClient->post("{$this->endpoint}/tools/{$toolId}", [
        'json' => [
          'arguments' => $parameters,
        ],
      ]);

      // We want to split this out into pure PHP in the future, thus not JSON.
      return json_decode($response->getBody()->getContents(), TRUE);
    }
    catch (GuzzleException $e) {
      throw $e;
    }
  }

}
