<?php

namespace Drupal\mcp_client\Plugin\AiFunctionCall;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Exception\AiToolsValidationException;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\mcp_client\MCPClient;
use Drupal\mcp_client\Plugin\AiFunctionCall\Derivative\McpPluginDeriver;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Plugin implementation of the action MCP function.
 */
#[FunctionCall(
  id: 'mcp_plugin',
  function_name: 'mcp_plugin',
  name: new TranslatableMarkup('MCP Plugin Wrapper'),
  description: '',
  deriver: McpPluginDeriver::class
)]
class McpPluginBase extends FunctionCallBase implements ExecutableFunctionCallInterface {

  /**
   * The output of the function call.
   *
   * @var string
   */
  protected string $output = '';

  /**
   * Constructs a FunctionCall plugin.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\ai\Utility\ContextDefinitionNormalizer $context_definition_normalizer
   *   The context definition normalizer service.
   * @param \Drupal\entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected ContextDefinitionNormalizer $context_definition_normalizer,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $context_definition_normalizer);
  }

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('ai.context_definition_normalizer'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    [$config_id, $tool] = explode(':', $this->getDerivativeId());

    try {
      $server = $this->entityTypeManager->getStorage('mcp_server')->load($config_id);
      if (!$server->get('status')) {
        throw new AiToolsValidationException('MCP server is not enabled.');
      }
      $tools = $server->get('tools');
      if (empty($tools[$tool])) {
        throw new AiToolsValidationException('MCP tool is not enabled.');
      }
      $enabled_tools = $server->get('enabled_tools');

      if (empty($enabled_tools)) {
        throw new AiToolsValidationException('MCP server has no enabled tools.');
      }
      if (!in_array($tool, $enabled_tools)) {
        throw new AiToolsValidationException('MCP tool is not enabled.');
      }
      $tool_info = $tools[$tool];
      $client = new MCPClient(
        $server->get('endpoint'),
      );
      $parameters = [];
      foreach ($tool_info['inputSchema']['properties'] as $key => $info) {
        // Get from context.
        $parameters[$key] = $this->getContextValue($key);
      }
      $response = $client->executeTool($tool_info['name'], $parameters);
      $this->output = Yaml::dump($response, 10, 2);
    }
    catch (\Exception $e) {
      throw new AiToolsValidationException('Invalid MCP server configuration: ' . $e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getReadableOutput(): string {
    return $this->output;
  }

}
