<?php

namespace Drupal\Tests\mcp_client\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\mcp_client\MCPClient;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Middleware;
use GuzzleHttp\Psr7\Response;

/**
 * Tests the MCPClient class.
 *
 * @group mcp_client
 * @coversDefaultClass \Drupal\mcp_client\MCPClient
 */
class MCPClientTest extends UnitTestCase {

  /**
   * The history container for requests.
   *
   * @var array
   */
  protected $historyContainer = [];

  /**
   * The MCP client under test.
   *
   * @var \Drupal\mcp_client\MCPClient
   */
  protected $mcpClient;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->historyContainer = [];
  }

  /**
   * Creates a mocked MCPClient with predefined responses.
   *
   * @param array $responses
   *   The mock responses to return.
   *
   * @return \Drupal\mcp_client\MCPClient
   *   The mocked client.
   */
  protected function createMockClient(array $responses): MCPClient {
    $mock = new MockHandler($responses);
    $handler = HandlerStack::create($mock);

    // Add history middleware to inspect requests.
    $history = Middleware::history($this->historyContainer);
    $handler->push($history);

    $client = new Client(['handler' => $handler]);
    return new MCPClient('https://example.com/api', $client);
  }

  /**
   * Tests the listTools method.
   *
   * @covers ::listTools
   */
  public function testListTools(): void {
    // Set up mock responses for two tools.
    $mockResponses = [
      new Response(200, [], json_encode([
        'tools' => [
          [
            'id' => 'text-generator',
            'name' => 'Text Generator',
            'description' => 'Generates text based on provided prompt',
          ],
          [
            'id' => 'image-analyzer',
            'name' => 'Image Analyzer',
            'description' => 'Analyzes images and extracts information',
          ],
        ],
      ])),
    ];

    $this->mcpClient = $this->createMockClient($mockResponses);
    $tools = $this->mcpClient->listTools();

    // Verify request was sent correctly.
    $this->assertCount(1, $this->historyContainer);
    $request = $this->historyContainer[0]['request'];
    $this->assertEquals('GET', $request->getMethod());
    $this->assertEquals('/api/tools', $request->getUri()->getPath());

    // Verify response was processed correctly.
    $this->assertCount(2, $tools);
    $this->assertEquals('text-generator', $tools[0]['id']);
    $this->assertEquals('Text Generator', $tools[0]['name']);
    $this->assertEquals('Generates text based on provided prompt', $tools[0]['description']);

    $this->assertEquals('image-analyzer', $tools[1]['id']);
    $this->assertEquals('Image Analyzer', $tools[1]['name']);
  }

  /**
   * Tests the executeTool method.
   *
   * @covers ::executeTool
   */
  public function testExecuteTool(): void {
    // Set up mock response for tool execution.
    $mockResponses = [
      new Response(200, [], json_encode([
        'success' => TRUE,
        'result' => 'This is generated text from the tool execution',
      ])),
    ];

    $this->mcpClient = $this->createMockClient($mockResponses);
    $params = [
      'prompt' => 'Generate a short poem about coding',
      'max_length' => 100,
    ];

    $result = $this->mcpClient->executeTool('text-generator', $params);

    // Verify request was sent correctly.
    $this->assertCount(1, $this->historyContainer);
    $request = $this->historyContainer[0]['request'];
    $this->assertEquals('POST', $request->getMethod());
    $this->assertEquals('/api/tools/text-generator/execute', $request->getUri()->getPath());

    // Verify request payload.
    $body = json_decode($this->historyContainer[0]['request']->getBody()->getContents(), TRUE);
    $this->assertEquals($params, $body['parameters']);

    // Verify response.
    $this->assertTrue($result['success']);
    $this->assertEquals('This is generated text from the tool execution', $result['result']);
  }

}
